#ifndef CMN_SCOPED_ACTION_H_INCLUDE
#define CMN_SCOPED_ACTION_H_INCLUDE
#include "boost\noncopyable.hpp"
#include "boost\function.hpp"

namespace cmn
{

class scoped_action :
    boost::noncopyable
{
    typedef boost::function<void()> function_type;
public:
    explicit scoped_action(const function_type& action) : 
        action_(action), released_(false) { }
    void release(){released_ = true;}
    ~scoped_action() 
    { 
        if(!released_)
            action_();
    }
private:
    bool released_;
    function_type action_;
};

}
#endif